from cpl_core.configuration import ConfigurationABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.mailing import EMailClientABC, EMailClientSettings
from cpl_translation import TranslatePipe
from flask import Response, jsonify, request

from bot_api.api import Api
from bot_api.filter.discord.server_select_criteria import ServerSelectCriteria
from bot_api.json_processor import JSONProcessor
from bot_api.logging.api_logger import ApiLogger
from bot_api.route.route import Route
from bot_api.service.discord_service import DiscordService
from bot_data.model.auth_role_enum import AuthRoleEnum


class ServerController:
    BasePath = f'/api/discord/server'

    def __init__(
            self,
            config: ConfigurationABC,
            env: ApplicationEnvironmentABC,
            logger: ApiLogger,
            t: TranslatePipe,
            api: Api,
            mail_settings: EMailClientSettings,
            mailer: EMailClientABC,
            discord_service: DiscordService
    ):
        self._config = config
        self._env = env
        self._logger = logger
        self._t = t
        self._api = api
        self._mail_settings = mail_settings
        self._mailer = mailer
        self._discord_service = discord_service

    @Route.get(f'{BasePath}/get/servers')
    @Route.authorize(role=AuthRoleEnum.admin)
    async def get_all_servers(self) -> Response:
        result = await self._discord_service.get_all_servers()
        result = result.select(lambda x: x.to_dict()).to_list()
        return jsonify(result)

    @Route.get(f'{BasePath}/get/servers-by-user')
    @Route.authorize
    async def get_all_servers_by_user(self) -> Response:
        result = await self._discord_service.get_all_servers_by_user()
        result = result.select(lambda x: x.to_dict()).to_list()
        return jsonify(result)

    @Route.post(f'{BasePath}/get/filtered')
    @Route.authorize
    async def get_filtered_servers(self) -> Response:
        dto: ServerSelectCriteria = JSONProcessor.process(ServerSelectCriteria, request.get_json(force=True, silent=True))
        result = await self._discord_service.get_filtered_servers_async(dto)
        result.result = result.result.select(lambda x: x.to_dict()).to_list()
        return jsonify(result.to_dict())

    @Route.get(f'{BasePath}/get/<id>')
    @Route.authorize
    async def get_server_by_id(self, id: int) -> Response:
        result = await self._discord_service.get_server_by_id_async(id).to_list()
        return jsonify(result.to_dict())
